﻿using System;
using System.IO;
using System.Net;
using System.Windows.Forms;

namespace DownloadWithProgress {
    public partial class MainForm : Form {
        public MainForm() {
            InitializeComponent();
        }


        private void Form_FormClosing(object sender, FormClosingEventArgs e) {
            if (bcwDownload.IsBusy) { bcwDownload.CancelAsync(); }
        }


        private void btnDownload_Click(object sender, EventArgs e) {
            btnDownload.Enabled = false;
            bcwDownload.RunWorkerAsync("http://www.jmedved.com/download/vhdattach210.exe");
        }

        private void btnCancel_Click(object sender, EventArgs e) {
            if (bcwDownload.IsBusy) { bcwDownload.CancelAsync(); }
        }

        private void bcwDownload_DoWork(object sender, System.ComponentModel.DoWorkEventArgs e) {
            string url = (string)e.Argument;

            var request = (HttpWebRequest)HttpWebRequest.Create(url);
            request.AllowAutoRedirect = true;
            request.Method = "GET";
            request.Proxy = HttpWebRequest.DefaultWebProxy;
            request.Proxy.Credentials = CredentialCache.DefaultNetworkCredentials;
            using (var response = (HttpWebResponse)request.GetResponse()) {
                var len = response.ContentLength;
                var buffer = new byte[256]; //256 bytes is nice round number :)
                using (var stream = response.GetResponseStream()) {
                    stream.ReadTimeout = 5000; //if one cannot get 256 bytes in 5 seconds, you might as well abandon all hope
                    using (var bytes = new MemoryStream()) {
                        while (bytes.Length < len) {
                            if (bcwDownload.CancellationPending) {
                                e.Cancel = true;
                                return;
                            }
                            var read = stream.Read(buffer, 0, buffer.Length);
                            if (read > 0) {
                                bytes.Write(buffer, 0, read);
                                bcwDownload.ReportProgress((int)(bytes.Length * 100 / len));
                            } else {
                                break;
                            }
                        }
                        if (bytes.Length != len) {
                            throw new WebException("File content is corrupted.");
                        } else {
                            e.Result = new DownloadResult(response.ResponseUri.Segments[response.ResponseUri.Segments.Length - 1], bytes.ToArray());
                        }
                    }
                }
            }
        }

        private void bcwDownload_ProgressChanged(object sender, System.ComponentModel.ProgressChangedEventArgs e) {
            prgDownload.Value = e.ProgressPercentage;
        }

        private void bcwDownload_RunWorkerCompleted(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e) {
            btnCancel.Enabled = false;
            if (e.Error != null) {
                MessageBox.Show(this, e.Error.Message, "Something", MessageBoxButtons.OK, MessageBoxIcon.Error);
            } else if (e.Cancelled) {
                MessageBox.Show(this, "User has canceled.", "Something", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            } else {
                var res = (DownloadResult)e.Result;
                var file = new FileInfo(res.FileName);
                using (var frm = new SaveFileDialog() { FileName = file.Name }) {
                    if (frm.ShowDialog(this) == DialogResult.OK) {
                        File.WriteAllBytes(frm.FileName, res.Bytes);
                        MessageBox.Show(this, "Done.", "Something", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    }
                }
            }
        }

    }
}
